@@ -4,9 +4,12 @@ use crate::{
     error::PERes,
     flush_checksum::{double_buffer_check, write_root_page},
     io::{read_u64, write_u64, InfallibleRead, InfallibleReadFormat},
+    persy::exp_from_content_size,
+    segment::{SEGMENTS_ROOT_PAGE_VERSION, SEGMENTS_ROOT_PAGE_VERSION_0},
 };
 use linked_hash_map::LinkedHashMap;
 use std::{
+    io::Write,
     sync::Mutex,
     time::{Duration, Instant},
 };
@@ -144,6 +147,13 @@ struct RootWriteInfo {
     buffer: Vec<u8>,
     version: u8,
 }
+
+#[derive(Clone, Default)]
+struct AddressData {
+    page: u64,
+    other_page: u64,
+}
+
 #[derive(Default)]
 struct RootPageHolder {
     page: u64,
@@ -175,7 +185,7 @@ struct Counter {
 struct FlushCount {
     free_list: Counter,
     journal: Counter,
-    address: Counter,
+    address: (Counter, AddressData),
 }
 
 // Root pages monitor to avoid to write a root page before the relative backup root page is not
@@ -412,24 +422,24 @@ impl Allocator {
             let page = self.disc.load_page(self.page)?.clone_write();
             let mut buffer = lock.write_list().to_vec();
             let holder = &mut monitor.free_list_holder;
-            self.write_root(page, holder, &mut buffer, ALLOCATOR_ROOT_PAGE_VERSION)?;
+            self.write_root(page.get_index(), holder, &mut buffer, ALLOCATOR_ROOT_PAGE_VERSION)?;
             // I do not do the disk sync here because is every time done by the caller.
             lock.to_flush = false;
         }
         Ok(())
     }
 
-    pub fn write_address_root(&self, root: Page, buffer: &mut [u8], version: u8) -> PERes<()> {
+    pub fn write_address_root(&self, root: u64, buffer: &mut [u8], version: u8) -> PERes<()> {
         let mut monitor = self.root_monitor.lock()?;
         self.write_root(root, &mut monitor.address_holder, buffer, version)
     }
     pub fn write_journal_root(&self, root: Page, buffer: &mut [u8], version: u8) -> PERes<()> {
         let mut monitor = self.root_monitor.lock()?;
-        self.write_root(root, &mut monitor.journal_holder, buffer, version)
+        self.write_root(root.get_index(), &mut monitor.journal_holder, buffer, version)
     }
 
-    fn write_root(&self, root: Page, holder: &mut RootPageHolder, buffer: &mut [u8], version: u8) -> PERes<()> {
-        holder.page = root.get_index();
+    fn write_root(&self, root: u64, holder: &mut RootPageHolder, buffer: &mut [u8], version: u8) -> PERes<()> {
+        holder.page = root;
         holder.version = version;
         holder.buffer = Some(Vec::from(buffer));
         holder.dirty = true;
@@ -442,10 +452,40 @@ impl Allocator {
         holder.flush_counter = order;
         Ok(())
     }
-    fn write_root_page_info(&self, mut info: RootWriteInfo, holder: &mut Counter) -> PERes<()> {
+    fn write_root_page_info(
+        &self,
+        mut info: RootWriteInfo,
+        holder: &mut Counter,
+        ad: Option<&mut AddressData>,
+    ) -> PERes<()> {
         let mut root = self.write_page(info.page)?;
         let last_flush = holder.flush_counter;
-        let order = write_root_page(&mut root, &mut info.buffer, info.version, last_flush)?;
+        let order = if let Some(bp) = ad {
+            let exp = exp_from_content_size(info.buffer.len() as u64);
+            let mut content_page = if bp.other_page == 0 {
+                self.allocate(exp)?
+            } else {
+                let mut page = self.write_page(bp.other_page)?;
+                if page.get_size_exp() != exp {
+                    self.free(bp.other_page)?;
+                    page = self.allocate(exp)?;
+                }
+                page
+            };
+            let content_page_id = content_page.get_index();
+            content_page.write_all(&info.buffer)?;
+            self.flush_page(content_page)?;
+
+            let mut root_buffer = [0; 19];
+            write_u64(&mut root_buffer[0..8], content_page_id);
+            write_u64(&mut root_buffer[8..16], bp.page);
+            let result = write_root_page(&mut root, &mut root_buffer, info.version, last_flush)?;
+            bp.other_page = bp.page;
+            bp.page = content_page_id;
+            result
+        } else {
+            write_root_page(&mut root, &mut info.buffer, info.version, last_flush)?
+        };
         self.flush_page(root)?;
         holder.flush_counter = order;
         Ok(())
@@ -469,9 +509,51 @@ impl Allocator {
             page,
             buffer_size,
             &mut monitor.address_holder,
-            &mut counter_monitor.address,
+            &mut counter_monitor.address.0,
         )
     }
+    pub fn create_address_root(&self, page: Page) -> PERes<()> {
+        let mut monitor = self.root_monitor.lock().unwrap();
+        monitor.address_holder.page = page.get_index();
+        monitor.address_holder.version = SEGMENTS_ROOT_PAGE_VERSION;
+        monitor.address_holder.buffer = Some(Vec::new());
+        monitor.address_holder.dirty = true;
+        Ok(())
+    }
+
+    pub fn read_address_buffer(&self, page: u64) -> PERes<Option<Vec<u8>>> {
+        let mut root = self.load_page(page)?;
+        match root.read_u8() {
+            SEGMENTS_ROOT_PAGE_VERSION_0 => {
+                let mut monitor = self.root_monitor.lock().unwrap();
+                monitor.address_holder.page = page;
+                monitor.address_holder.dirty = false;
+                monitor.address_holder.version = SEGMENTS_ROOT_PAGE_VERSION_0;
+                let mut counter_monitor = self.flush_count.lock().unwrap();
+                let mut buffer_0 = vec![0; 19];
+                let mut buffer_1 = vec![0; 19];
+                InfallibleRead::read_exact(&mut root, &mut buffer_0);
+                InfallibleRead::read_exact(&mut root, &mut buffer_1);
+                let (flush, first) = double_buffer_check(&buffer_0, &buffer_1);
+                let buffer = if first { buffer_0 } else { buffer_1 };
+                counter_monitor.address.0.flush_counter = flush;
+                let page_id = read_u64(&buffer[0..8]);
+                let other_page_id = read_u64(&buffer[8..16]);
+                counter_monitor.address.1.page = page_id;
+                counter_monitor.address.1.other_page = other_page_id;
+                if page_id != 0 {
+                    let page = self.load_page(page_id)?;
+                    let buffer = page.content();
+                    monitor.address_holder.buffer = Some(buffer.clone());
+                    Ok(Some(buffer))
+                } else {
+                    monitor.address_holder.buffer = None;
+                    Ok(None)
+                }
+            }
+            _ => panic!("version not supported"),
+        }
+    }
 
     fn read_root_page_int(
         page: &mut ReadPage,
@@ -511,13 +593,14 @@ impl Allocator {
         {
             let mut fm = self.flush_count.lock()?;
             if let Some(info) = free_list_data {
-                self.write_root_page_info(info, &mut fm.free_list)?;
+                self.write_root_page_info(info, &mut fm.free_list, None)?;
             }
             if let Some(info) = journal_data {
-                self.write_root_page_info(info, &mut fm.journal)?;
+                self.write_root_page_info(info, &mut fm.journal, None)?;
             }
             if let Some(info) = address_data {
-                self.write_root_page_info(info, &mut fm.address)?;
+                let (counter, data) = &mut fm.address;
+                self.write_root_page_info(info, counter, Some(data))?;
             }
             self.disc.sync()?;
         }
