@@ -19,6 +19,8 @@ mod tests;
 pub const PAGE_METADATA_SIZE: u32 = 3;
 const SIZE_OFFSET: usize = 0;
 const FLAGS_OFFSET: usize = SIZE_OFFSET + 1;
+//Data and next offset are the same because are used in different states.
+const DATA_OFFSET: usize = FLAGS_OFFSET + 1;
 const NEXT_OFFSET: usize = FLAGS_OFFSET + 1;
 const PREV_OFFSET: usize = NEXT_OFFSET + 8;
 pub trait UpdateList {
@@ -102,6 +104,9 @@ impl ReadPage {
     pub fn arc_slice(&self, len: usize) -> ArcSliceRead {
         ArcSliceRead::new(self.buff.clone(), self.pos, self.pos + len)
     }
+    pub fn content(&self) -> Vec<u8> {
+        self.buff[DATA_OFFSET..self.buff.len() - 1].to_vec()
+    }
 }
 
 impl Read for ReadPage {
