@@ -309,6 +309,8 @@ impl PersyImpl {
         for (_, (_, tx, _)) in recover.transactions.iter_mut() {
             tx.recover_rollback(self)?;
         }
+        self.address.recompute_last_pages()?;
+        self.address.flush_segments()?;
         allocator.disc_sync()?;
         self.journal.clear_in_queue()?;
         allocator.trim_free_at_end()?;
@@ -1082,7 +1084,7 @@ impl PersyImpl {
         match sync_mode {
             SyncMode::Sync => {
                 let allocator = self.allocator();
-                allocator.disc().sync()?;
+                allocator.disc_sync()?;
                 self.journal().clear_in_queue()?;
             }
             SyncMode::BackgroundSync => unreachable!(),
