@@ -7,8 +7,7 @@ use crate::{
     device::{Page, PageOps},
     error::PERes,
     id::{RecRef, SegmentId},
-    io::{read_u64, write_u64, InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat},
-    persy::exp_from_content_size,
+    io::{ArcSliceRead, InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat},
 };
 use std::{
     collections::{hash_map::DefaultHasher, HashMap},
@@ -18,8 +17,8 @@ use std::{
     vec,
 };
 
-const SEGMENTS_ROOT_PAGE_VERSION_0: u8 = 0;
-const SEGMENTS_ROOT_PAGE_VERSION: u8 = SEGMENTS_ROOT_PAGE_VERSION_0;
+pub(crate) const SEGMENTS_ROOT_PAGE_VERSION_0: u8 = 0;
+pub(crate) const SEGMENTS_ROOT_PAGE_VERSION: u8 = SEGMENTS_ROOT_PAGE_VERSION_0;
 
 pub struct AllocatedSegmentPage {
     pub new_page: u64,
@@ -329,8 +328,6 @@ impl<T: InfallibleRead + InfallibleWrite + PageOps> SegmentPage for T {
 
 pub struct Segments {
     pub root_page: u64,
-    content_page: u64,
-    other_page: u64,
     pub segments: HashMap<SegmentId, Segment>,
     pub segments_id: HashMap<String, SegmentId>,
     pub temp_segments: HashMap<SegmentId, Segment>,
@@ -349,23 +346,11 @@ pub fn segment_hash(segment: &str) -> u64 {
 
 impl Segments {
     pub fn new(root_page: u64, allocator: &Arc<Allocator>) -> PERes<Segments> {
-        let page_id;
-        let other_page_id;
-        {
-            let mut root = allocator.load_page(root_page)?;
-            match root.read_u8() {
-                SEGMENTS_ROOT_PAGE_VERSION_0 => {
-                    let buffer = allocator.read_root_address(&mut root, 19);
-                    page_id = read_u64(&buffer[0..8]);
-                    other_page_id = read_u64(&buffer[8..16]);
-                }
-                _ => panic!("version not supported"),
-            }
-        }
+        let read_buffer = allocator.read_address_buffer(root_page)?;
         let mut segments = HashMap::new();
         let mut segments_id = HashMap::new();
-        if page_id != 0 {
-            let mut page = allocator.load_page(page_id)?;
+        if let Some(buffer) = read_buffer {
+            let mut page = ArcSliceRead::new_vec(buffer);
             loop {
                 let flag = page.read_u8();
                 if flag == 1 {
@@ -382,8 +367,6 @@ impl Segments {
         }
         Ok(Segments {
             root_page,
-            content_page: page_id,
-            other_page: other_page_id,
             segments,
             segments_id,
             temp_segments: HashMap::new(),
@@ -392,10 +375,7 @@ impl Segments {
     }
 
     pub fn init(root: Page, allocator: &Allocator) -> PERes<()> {
-        let mut buffer = [0; 19];
-        write_u64(&mut buffer[0..8], 0);
-        write_u64(&mut buffer[8..16], 0);
-        allocator.write_address_root(root, &mut buffer, SEGMENTS_ROOT_PAGE_VERSION)?;
+        allocator.create_address_root(root)?;
         Ok(())
     }
 
@@ -532,32 +512,7 @@ impl Segments {
             segment.write(&mut buffer);
         }
         buffer.write_u8(0);
-        let exp = exp_from_content_size(buffer.len() as u64);
-        let other_page_id = self.content_page;
-        let mut content_page_id = self.other_page;
-        {
-            let mut content_page = if content_page_id == 0 {
-                allocator.allocate(exp)?
-            } else {
-                let mut page = allocator.write_page(content_page_id)?;
-                if page.get_size_exp() != exp {
-                    allocator.free(content_page_id)?;
-                    page = allocator.allocate(exp)?;
-                }
-                page
-            };
-            content_page_id = content_page.get_index();
-            content_page.write_all(&buffer);
-            allocator.flush_page(content_page)?;
-        }
-
-        let mut root_buffer = [0; 19];
-        write_u64(&mut root_buffer[0..8], content_page_id);
-        write_u64(&mut root_buffer[8..16], other_page_id);
-        let root = allocator.write_page(self.root_page)?;
-        allocator.write_address_root(root, &mut root_buffer, SEGMENTS_ROOT_PAGE_VERSION)?;
-        self.content_page = content_page_id;
-        self.other_page = other_page_id;
+        allocator.write_address_root(self.root_page, &mut buffer, SEGMENTS_ROOT_PAGE_VERSION)?;
         Ok(())
     }
 
