@@ -2,9 +2,8 @@ use crate::{
     allocator::Allocator,
     error::PERes,
     id::{RecRef, SegmentId},
-    journal::{Journal, JournalId},
+    journal::{records::FreedPage, Journal, JournalId},
     segment::SegmentPageIterator,
-    transaction_impl::FreedPage,
 };
 use std::{
     cmp::Ordering,
@@ -47,14 +46,14 @@ impl SnapshotEntry {
 }
 
 #[derive(Clone, Debug)]
-pub struct SegmentSnapshop {
+pub struct SegmentSnapshot {
     name: String,
     id: SegmentId,
     first_page: u64,
 }
-impl SegmentSnapshop {
-    pub fn new(name: &str, id: SegmentId, first_page: u64) -> SegmentSnapshop {
-        SegmentSnapshop {
+impl SegmentSnapshot {
+    pub fn new(name: &str, id: SegmentId, first_page: u64) -> SegmentSnapshot {
+        SegmentSnapshot {
             name: name.to_string(),
             id,
             first_page,
@@ -68,8 +67,8 @@ pub struct SnapshotData {
     journal_id: Option<JournalId>,
     entries: Option<Vec<SnapshotEntry>>,
     freed_pages: Option<Vec<FreedPage>>,
-    segments: Option<HashMap<SegmentId, SegmentSnapshop>>,
-    segments_name: Option<HashMap<String, SegmentSnapshop>>,
+    segments: Option<HashMap<SegmentId, SegmentSnapshot>>,
+    segments_name: Option<HashMap<String, SegmentSnapshot>>,
     reference_count: u32,
 }
 
@@ -120,7 +119,7 @@ pub struct InternalSnapshots {
 impl InternalSnapshots {
     fn next_snapshot_id(&mut self) -> SnapshotId {
         let snapshot_id = self.snapshot_sequence;
-        // This should be the default behaviour of the u64 anyway, let's hope the compiler trows
+        // This should be the default behavior of the u64 anyway, let's hope the compiler trows
         // this away
         if self.snapshot_sequence == u64::MAX {
             self.snapshot_sequence = 0;
@@ -330,7 +329,7 @@ impl Snapshots {
         Ok(snapshot_id)
     }
 
-    pub fn fill_segments(&self, snapshot_id: SnapshotId, segments: &[SegmentSnapshop]) -> PERes<()> {
+    pub fn fill_segments(&self, snapshot_id: SnapshotId, segments: &[SegmentSnapshot]) -> PERes<()> {
         let mut segments_id = HashMap::new();
         let mut segments_name = HashMap::new();
         for segment in segments {
@@ -461,11 +460,10 @@ pub fn release_snapshot(id: SnapshotId, snapshots: &Snapshots, allocator: &Alloc
 
 #[cfg(test)]
 mod tests {
-    use super::{search, Change, EntryCase, RecordVersion, SegmentSnapshop, SnapshotEntry, Snapshots};
+    use super::{search, Change, EntryCase, RecordVersion, SegmentSnapshot, SnapshotEntry, Snapshots};
     use crate::{
         id::{RecRef, SegmentId},
-        journal::JournalId,
-        transaction_impl::FreedPage,
+        journal::{records::FreedPage, JournalId},
     };
     use std::cmp::Ordering;
 
@@ -524,7 +522,7 @@ mod tests {
     }
 
     #[test]
-    fn add_and_read_mutliple_tx() {
+    fn add_and_read_multiple_tx() {
         let snap = Snapshots::new();
         let mut txs = Vec::new();
         for t in 0..5 {
@@ -642,8 +640,8 @@ mod tests {
             let id = snap.read_snapshot().unwrap();
             let mut segments = Vec::new();
             let seg_id = SegmentId::new(i);
-            segments.push(SegmentSnapshop::new("one", seg_id, i.into()));
-            segments.push(SegmentSnapshop::new("two", seg_id, i.into()));
+            segments.push(SegmentSnapshot::new("one", seg_id, i.into()));
+            segments.push(SegmentSnapshot::new("two", seg_id, i.into()));
             snap.fill_segments(id, &segments).expect("fill works");
             snaps.push(id);
         }
@@ -662,7 +660,7 @@ mod tests {
         let id = snap.read_snapshot().unwrap();
         let mut segments = Vec::new();
         let seg_id = SegmentId::new(10);
-        segments.push(SegmentSnapshop::new("one", seg_id, 10));
+        segments.push(SegmentSnapshot::new("one", seg_id, 10));
         snap.fill_segments(id, &segments).expect("fill works");
         let solved = snap.solve_segment_id(id, "one").expect("solve id correctly");
         assert_eq!(solved, Some(seg_id));
@@ -683,8 +681,8 @@ mod tests {
             let id = snap.read_snapshot().unwrap();
             let mut segments = Vec::new();
             let seg_id = SegmentId::new(i);
-            segments.push(SegmentSnapshop::new("one", seg_id, i.into()));
-            segments.push(SegmentSnapshop::new("two", seg_id, i.into()));
+            segments.push(SegmentSnapshot::new("one", seg_id, i.into()));
+            segments.push(SegmentSnapshot::new("two", seg_id, i.into()));
             snap.fill_segments(id, &segments).expect("fill works");
             snaps.push(id);
         }
